{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "import numpy as np\n",
    "import pandas as pd\n",
    "import copy\n",
    "import seaborn as sns\n",
    "import matplotlib.pyplot as plt"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "class Elevator(object):\n",
    "    \"\"\"An Elevator is a generic class that can move one floor up and down and\n",
    "    let the passengers off and on simultaneously with the help of a convenience function called \"open_door\".\n",
    "    \"\"\"\n",
    "\n",
    "    def __init__(self, building, capacity = 10):\n",
    "        \"\"\" Initializes an instance of a class with a certain capacity identified by the user.\n",
    "        Each object is characterized by the number of passengers, its current floor, the building\n",
    "        where it is situated, and a list of passengers inside the elevator.\n",
    "        \"\"\"\n",
    "        self.capacity = capacity   # number of people elevator can hold\n",
    "        self.current_floor = 1     # elevator starts at ground floor: 1\n",
    "        self.building = building\n",
    "        self.num_passengers = 0\n",
    "\n",
    "        # self.passengers is a dictionary from a destination floor number\n",
    "        # to a list of Passenger objects (the Passengers that need to get off at floor i)\n",
    "        self.passengers = { \n",
    "            i : [] for i in range(1, building.num_floors + 1)\n",
    "        }\n",
    "\n",
    "\n",
    "    def move(self, direction):\n",
    "        \"\"\"Moves the elevator one floor up or down. \"\"\"\n",
    "        self.current_floor += direction # direction is integer +1 or -1\n",
    "\n",
    "    def let_off(self, passenger, timestep = None):\n",
    "        \"\"\"Removes `passenger` from the elevator and adds them to the building.\n",
    "        If `passenger` has arrived on their destination floor, add them to `arrived_passengers`.\n",
    "        Raises an error if `passenger` is not on the elevator.\n",
    "        \"\"\"\n",
    "        try:\n",
    "            self.passengers[passenger.end_floor].remove(passenger)\n",
    "        except:\n",
    "            raise(\"Passenger not found on elevator!\")\n",
    "        self.num_passengers -= 1\n",
    "\n",
    "        if passenger.end_floor == self.current_floor:\n",
    "            self.building.arrived_passengers.append(passenger)\n",
    "        else:\n",
    "            self.building.waiting_passengers[self.current_floor].append(passenger)\n",
    "            self.building.num_passengers += 1\n",
    "\n",
    "        if timestep is not None:\n",
    "            passenger.travel_time += timestep - passenger.last_boarding\n",
    "            passenger.last_deboarding = timestep\n",
    "\n",
    "        return\n",
    "\n",
    "    def let_on(self, passenger, timestep = None):\n",
    "        \"\"\"Adds `passenger` to the elevator from the current floor\n",
    "        Raises an error if `passenger` is not on the elevator's current_floor\n",
    "        \"\"\"\n",
    "        try:\n",
    "            self.building.waiting_passengers[self.current_floor].remove(passenger)\n",
    "        except:\n",
    "            raise(\"Passenger not found on current floor!\")\n",
    "        self.building.num_passengers -= 1\n",
    "\n",
    "        self.passengers[passenger.end_floor].append(passenger)\n",
    "        self.num_passengers += 1\n",
    "\n",
    "        if timestep is not None:\n",
    "            passenger.wait_time += timestep - passenger.last_deboarding\n",
    "            passenger.last_boarding = timestep\n",
    "\n",
    "        return\n",
    "\n",
    "    def open_doors(self, timestep = None):\n",
    "        \"\"\"A convenience function for letting people on and off floors simultaneously\"\"\"\n",
    "\n",
    "        # passengers get off\n",
    "        for passenger in self.passengers[self.current_floor]:\n",
    "            self.let_off(passenger, timestep)\n",
    "\n",
    "        # passengers get on\n",
    "        floor_lst = self.building.waiting_passengers[self.current_floor]\n",
    "        while self.capacity > self.num_passengers and floor_lst:\n",
    "            passenger = floor_lst[-1]\n",
    "            self.let_on(passenger, timestep)\n",
    "\n",
    "        return\n",
    "\n",
    "    def not_empty(self):\n",
    "        \"\"\"Returns True if elevator is not empty.\"\"\"\n",
    "        return self.num_passengers > 0\n",
    "\n",
    "class Building(object):\n",
    "    \"\"\"The building generates and stores the passengers waiting for the elevator.\"\"\"\n",
    "\n",
    "    def __init__(self, num_floors, num_passengers):\n",
    "        \"\"\"By default requires user specified number of floors and passengers.\"\"\"\n",
    "        self.num_floors = num_floors\n",
    "        self.arrived_passengers = []\n",
    "        self.waiting_passengers = { \n",
    "            i : [] for i in range(1, num_floors + 1)\n",
    "        }\n",
    "        self.num_passengers = 0\n",
    "        self.add_passengers_randomly(num_passengers)\n",
    "\n",
    "    def add_passengers_randomly(self, number):\n",
    "        \"\"\"Generates random passengers added to each floor.\"\"\"\n",
    "        for i in range(number):\n",
    "            start = np.random.randint(1, self.num_floors + 1)\n",
    "            end = start\n",
    "            while end == start:\n",
    "                end = np.random.randint(1, self.num_floors + 1)\n",
    "            passenger = Passenger(start,end)\n",
    "            self.waiting_passengers[passenger.start_floor].append(passenger)\n",
    "        self.num_passengers += number\n",
    "\n",
    "    def not_empty(self):\n",
    "        \"\"\"Returns True if building is not empty.\"\"\"\n",
    "        return self.num_passengers > 0\n",
    "\n",
    "class Passenger(object):\n",
    "    \"\"\"Initiates passengers with start and end floor, called in building class.\"\"\"\n",
    "\n",
    "    def __init__(self, start_floor, end_floor):\n",
    "        \"\"\"Initializes variables for keeping track of wait_ and travel_time\n",
    "           Allows for multiple boardings/deboardings from the elevator\n",
    "        \"\"\"\n",
    "        self.start_floor = start_floor\n",
    "        self.end_floor = end_floor\n",
    "        self.last_boarding = 0\n",
    "        self.last_deboarding = 0\n",
    "        self.wait_time = 0\n",
    "        self.travel_time = 0"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "94\n",
      "90\n",
      "89\n",
      "88\n",
      "87\n",
      "87\n",
      "86\n",
      "86\n",
      "86\n",
      "85\n",
      "84\n",
      "83\n",
      "81\n",
      "80\n",
      "79\n",
      "79\n",
      "79\n",
      "78\n",
      "77\n",
      "76\n",
      "76\n",
      "75\n",
      "75\n",
      "75\n",
      "75\n",
      "75\n",
      "74\n",
      "73\n",
      "72\n",
      "72\n",
      "71\n",
      "71\n",
      "71\n",
      "70\n",
      "70\n",
      "68\n",
      "66\n",
      "65\n",
      "65\n",
      "64\n",
      "63\n",
      "63\n",
      "63\n",
      "63\n",
      "62\n",
      "62\n",
      "62\n",
      "62\n",
      "62\n",
      "62\n",
      "61\n",
      "61\n",
      "60\n",
      "59\n",
      "59\n",
      "58\n",
      "57\n",
      "55\n",
      "55\n",
      "55\n",
      "55\n",
      "55\n",
      "55\n",
      "55\n",
      "54\n",
      "54\n",
      "53\n",
      "53\n",
      "53\n",
      "52\n",
      "52\n",
      "52\n",
      "52\n",
      "51\n",
      "51\n",
      "49\n",
      "49\n",
      "48\n",
      "48\n",
      "48\n",
      "48\n",
      "46\n",
      "45\n",
      "45\n",
      "45\n",
      "45\n",
      "44\n",
      "43\n",
      "43\n",
      "42\n",
      "41\n",
      "41\n",
      "41\n",
      "41\n",
      "40\n",
      "40\n",
      "38\n",
      "38\n",
      "38\n",
      "38\n",
      "38\n",
      "38\n",
      "38\n",
      "37\n",
      "36\n",
      "36\n",
      "35\n",
      "35\n",
      "34\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "31\n",
      "29\n",
      "28\n",
      "27\n",
      "27\n",
      "26\n",
      "26\n",
      "25\n",
      "25\n",
      "25\n",
      "24\n",
      "24\n",
      "24\n",
      "24\n",
      "24\n",
      "23\n",
      "21\n",
      "21\n",
      "21\n",
      "20\n",
      "20\n",
      "19\n",
      "19\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "17\n",
      "16\n",
      "14\n",
      "11\n",
      "11\n",
      "11\n",
      "11\n",
      "10\n",
      "9\n",
      "8\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "4\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "3\n",
      "1\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "94\n",
      "94\n",
      "90\n",
      "90\n",
      "89\n",
      "89\n",
      "88\n",
      "88\n",
      "87\n",
      "87\n",
      "87\n",
      "86\n",
      "86\n",
      "86\n",
      "86\n",
      "85\n",
      "85\n",
      "84\n",
      "84\n",
      "83\n",
      "83\n",
      "81\n",
      "80\n",
      "80\n",
      "79\n",
      "79\n",
      "78\n",
      "78\n",
      "78\n",
      "78\n",
      "77\n",
      "77\n",
      "76\n",
      "76\n",
      "75\n",
      "74\n",
      "74\n",
      "74\n",
      "73\n",
      "73\n",
      "73\n",
      "73\n",
      "73\n",
      "73\n",
      "73\n",
      "72\n",
      "72\n",
      "71\n",
      "70\n",
      "70\n",
      "70\n",
      "69\n",
      "69\n",
      "68\n",
      "68\n",
      "68\n",
      "67\n",
      "67\n",
      "67\n",
      "67\n",
      "65\n",
      "65\n",
      "63\n",
      "62\n",
      "62\n",
      "62\n",
      "59\n",
      "58\n",
      "58\n",
      "58\n",
      "58\n",
      "58\n",
      "58\n",
      "58\n",
      "58\n",
      "57\n",
      "56\n",
      "56\n",
      "56\n",
      "56\n",
      "56\n",
      "55\n",
      "55\n",
      "54\n",
      "54\n",
      "53\n",
      "52\n",
      "52\n",
      "52\n",
      "51\n",
      "50\n",
      "50\n",
      "49\n",
      "49\n",
      "49\n",
      "48\n",
      "48\n",
      "47\n",
      "47\n",
      "46\n",
      "46\n",
      "46\n",
      "46\n",
      "43\n",
      "43\n",
      "43\n",
      "43\n",
      "43\n",
      "43\n",
      "43\n",
      "43\n",
      "43\n",
      "42\n",
      "42\n",
      "42\n",
      "41\n",
      "41\n",
      "40\n",
      "39\n",
      "39\n",
      "39\n",
      "38\n",
      "37\n",
      "37\n",
      "36\n",
      "35\n",
      "35\n",
      "35\n",
      "34\n",
      "34\n",
      "34\n",
      "34\n",
      "34\n",
      "34\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "33\n",
      "32\n",
      "32\n",
      "31\n",
      "31\n",
      "28\n",
      "28\n",
      "27\n",
      "27\n",
      "26\n",
      "25\n",
      "25\n",
      "25\n",
      "25\n",
      "23\n",
      "22\n",
      "22\n",
      "22\n",
      "22\n",
      "20\n",
      "19\n",
      "19\n",
      "19\n",
      "18\n",
      "18\n",
      "16\n",
      "15\n",
      "15\n",
      "15\n",
      "15\n",
      "15\n",
      "15\n",
      "15\n",
      "15\n",
      "15\n",
      "14\n",
      "14\n",
      "13\n",
      "13\n",
      "12\n",
      "12\n",
      "12\n",
      "11\n",
      "11\n",
      "11\n",
      "11\n",
      "11\n",
      "11\n",
      "11\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "9\n",
      "8\n",
      "8\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "7\n",
      "3\n",
      "3\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n",
      "0\n"
     ]
    }
   ],
   "source": [
    "# Main Program\n",
    "\n",
    "def display_state_and_collect_analytics(elevator, building, analytics):\n",
    "    \"\"\"Displays the current elevator and building state.\n",
    "       Collects relevant data about the performance of the elevator.\n",
    "    \"\"\"\n",
    "    print building.num_passengers\n",
    "\n",
    "def run_simulation(elevator,building,strategy,log_progress=True):\n",
    "    \"\"\"Runs the simulation for a given strategy.\n",
    "       Continues to call the given strategy until\n",
    "       all passengers are processed.\n",
    "    \"\"\"\n",
    "    is_moving_up = True\n",
    "    timestep = 0\n",
    "    analytics = {'waiting_times':[]}\n",
    "    while building.not_empty() or elevator.not_empty():\n",
    "        timestep,is_moving_up = strategy(elevator,building,timestep,is_moving_up)\n",
    "        if log_progress:\n",
    "            display_state_and_collect_analytics(elevator, building, analytics)\n",
    "    analytics['tot_time'] = timestep\n",
    "    analytics['arrived_passengers'] = building.arrived_passengers\n",
    "\n",
    "    return analytics\n",
    "\n",
    "def basic_controller(elevator, building,timestep,is_moving_up):\n",
    "    \"\"\"Implements basic elevator strategy.\n",
    "       Elevator goes up and down the building,\n",
    "       stopping at every floor along the way.\n",
    "    \"\"\"\n",
    "    elevator.open_doors(timestep = timestep)\n",
    "    timestep += 1\n",
    "\n",
    "    # switch directions if the elevator reaches the top or bottom of the building\n",
    "    if elevator.current_floor == building.num_floors:\n",
    "        is_moving_up = False\n",
    "    elif elevator.current_floor == 1:\n",
    "        is_moving_up = True\n",
    "\n",
    "    # elevator moves up or down\n",
    "    elevator.move(1 if is_moving_up else -1)\n",
    "\n",
    "    timestep += 1\n",
    "\n",
    "    return(timestep,is_moving_up)\n",
    "\n",
    "def secondary_controller(elevator, building, timestep,is_moving_up):\n",
    "    \"\"\"Implements an elevator strategy where the\n",
    "       elevator goes up and down the building,\n",
    "       stopping at every floor where someone needs to get off or on.\n",
    "    \"\"\"\n",
    "    if elevator.current_floor == building.num_floors:\n",
    "        is_moving_up = False\n",
    "    elif elevator.current_floor == 1:\n",
    "        is_moving_up = True\n",
    "\n",
    "    on_floor = building.waiting_passengers[elevator.current_floor]\n",
    "    off_floor = elevator.passengers[elevator.current_floor]\n",
    "    space = True if elevator.num_passengers < elevator.capacity else False\n",
    "\n",
    "    if (off_floor) or (on_floor and space):\n",
    "        elevator.open_doors(timestep = timestep)\n",
    "    else:\n",
    "        # switch directions if the elevator reaches the top or bottom of the building\n",
    "        elevator.move(1 if is_moving_up else -1)\n",
    "\n",
    "    # elevator moves up or down\n",
    "    timestep += 1\n",
    "    return(timestep,is_moving_up)\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "\n",
    "    # create building object with random passenger set up\n",
    "    building1 = Building(num_floors=20, num_passengers=100)\n",
    "\n",
    "    # an identical copy of the building\n",
    "    building2 = copy.deepcopy(building1)\n",
    "\n",
    "    # two elevators, each with its own associated building\n",
    "    elevator1 = Elevator(building1, capacity=10)\n",
    "    elevator2 = Elevator(building2, capacity=10)\n",
    "\n",
    "    # run simulations with each of the two strategies\n",
    "    results1 = run_simulation(elevator1,building1,basic_controller)\n",
    "    results2 = run_simulation(elevator2,building2,secondary_controller)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "collapsed": false
   },
   "source": [
    "## Efficiency Comparison"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "def extract_times(results,strategy):\n",
    "    \"\"\"Extract efficiency metrics from stored passenger objects.\"\"\"\n",
    "\n",
    "    waited = [] # times waiting for elevator\n",
    "    travelled = [] # times on elevator\n",
    "    totals = [] # total times travelled\n",
    "\n",
    "    # add every passengers data to aggregate lists\n",
    "    for passenger in results['arrived_passengers']:\n",
    "        waited.append(passenger.wait_time)\n",
    "        travelled.append(passenger.travel_time)\n",
    "        totals.append(passenger.wait_time+passenger.travel_time)\n",
    "\n",
    "    # update dictionary with extracted data\n",
    "    strategy[\"runtimes\"].append(results[\"tot_time\"])\n",
    "    strategy[\"waits\"].append(waited)\n",
    "    strategy[\"travels\"].append(travelled)\n",
    "    strategy[\"totals\"].append(totals)\n",
    "\n",
    "\n",
    "def compare_strategies(strategies,runs=100):\n",
    "    \"\"\"Run simulation 'runs' times for each strategy and extract efficiency metrics.\"\"\"\n",
    "\n",
    "    metrics = []\n",
    "    # create a metric dictionary for each strategy\n",
    "    for strategy in strategies:\n",
    "        metrics.append({\"runtimes\":[],\"waits\":[],\"travels\":[],\"totals\":[]})\n",
    "\n",
    "    # run the simulation for each strategy, 'runs' times\n",
    "    for sim_run in range(runs):\n",
    "        basic_building = Building(num_floors=20, num_passengers=100)\n",
    "        buildings = []\n",
    "        elevators = []\n",
    "        results = []\n",
    "        for i,strategy in enumerate(strategies):\n",
    "            building = copy.deepcopy(basic_building)\n",
    "            elevator = Elevator(building, capacity=10)\n",
    "            results = run_simulation(elevator,building,strategy,log_progress=False)\n",
    "            extract_times(results,metrics[i])\n",
    "\n",
    "    return(metrics)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "# collect efficiency measures for each strategy developed\n",
    "strategies = [basic_controller,secondary_controller]\n",
    "strategy1,strategy2 = compare_strategies(strategies,1000)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Strategy 1</th>\n",
       "      <th>Strategy 2</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>count</th>\n",
       "      <td>1000.000000</td>\n",
       "      <td>1000.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>mean</th>\n",
       "      <td>451.926000</td>\n",
       "      <td>279.687000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>std</th>\n",
       "      <td>41.834683</td>\n",
       "      <td>19.346869</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>min</th>\n",
       "      <td>324.000000</td>\n",
       "      <td>229.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>25%</th>\n",
       "      <td>420.000000</td>\n",
       "      <td>260.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>50%</th>\n",
       "      <td>458.000000</td>\n",
       "      <td>279.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>75%</th>\n",
       "      <td>474.000000</td>\n",
       "      <td>296.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>max</th>\n",
       "      <td>610.000000</td>\n",
       "      <td>341.000000</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ]
     },
     "execution_count": 6,
     "metadata": {
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# simple tabular summary of simulation runtimes under each strategy\n",
    "\n",
    "pd.DataFrame([strategy1[\"runtimes\"],strategy2[\"runtimes\"]],\n",
    "             index=['Strategy 1','Strategy 2']).T.describe()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "image/png": "0e30a34ceed5ef8ab61680e94a49382b185b990e"
     },
     "execution_count": 7,
     "metadata": {
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "plt.figure(figsize=(8,4))\n",
    "sns.distplot(strategy1[\"runtimes\"],label='Strategy 1',kde=False)\n",
    "sns.distplot(strategy2[\"runtimes\"],label='Strategy 2',kde=False)\n",
    "plt.title('Histograms of Simulation Runtimes')\n",
    "plt.ylabel('Number of Simulations')\n",
    "plt.xlabel('Runtime (time to process all passengers)')\n",
    "plt.legend()\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "def all_waiting_times(times):\n",
    "    \"\"\"Simple helper function to merge list of lists.\"\"\"\n",
    "    return([time for simulation in times for time in simulation])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "image/png": "bfb567926dbc578d5e86c4fc3066ae2f495ab3d0"
     },
     "execution_count": 9,
     "metadata": {
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "plt.figure(figsize=(8,4))\n",
    "sns.distplot(all_waiting_times(strategy1['waits']),label='Strategy 1',kde=False,bins=10)\n",
    "sns.distplot(all_waiting_times(strategy2['waits']),label='Strategy 2',kde=False,bins=10)\n",
    "plt.title('Histograms of Individual Waiting Times')\n",
    "plt.ylabel('Number of Passengers')\n",
    "plt.xlabel('Waiting Time (time waiting for elevator)')\n",
    "plt.legend()\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "image/png": "582ec1d04b8c04e222cd0dea2f6cab8feb3946c1"
     },
     "execution_count": 10,
     "metadata": {
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "plt.figure(figsize=(8,4))\n",
    "sns.distplot(all_waiting_times(strategy1['travels']),label='Strategy 1',kde=False,bins=10)\n",
    "sns.distplot(all_waiting_times(strategy2['travels']),label='Strategy 2',kde=False,bins=10)\n",
    "plt.title('Histograms of Individual Travel Times')\n",
    "plt.ylabel('Number of Passengers')\n",
    "plt.xlabel('Travel Time (time on elevator)')\n",
    "plt.legend()\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "image/png": "6afceb3216d2bdfba103b06258ebd2f1b7100557"
     },
     "execution_count": 11,
     "metadata": {
     },
     "output_type": "execute_result"
    }
   ],
   "source": [
    "plt.figure(figsize=(8,4))\n",
    "sns.distplot(all_waiting_times(strategy1['totals']),label='Strategy 1',kde=False,bins=10)\n",
    "sns.distplot(all_waiting_times(strategy2['totals']),label='Strategy 2',kde=False,bins=10)\n",
    "plt.title('Histograms of Individual Time in Simulation')\n",
    "plt.ylabel('Number of Passengers')\n",
    "plt.xlabel('Total Time (time in simulation)')\n",
    "plt.legend()\n",
    "plt.show()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 0,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 2 (SageMath)",
   "language": "python",
   "name": "python2"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 2
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython2",
   "version": "2.7.14"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}